"""abstractruntime.evidence

Runtime-owned evidence capture and indexing.
"""

from .recorder import EvidenceRecorder, DEFAULT_EVIDENCE_TOOL_NAMES

__all__ = ["EvidenceRecorder", "DEFAULT_EVIDENCE_TOOL_NAMES"]


